import util from 'util'
import config from '../globals'

module.exports.log = function () {
  console.log.apply(null, arguments)
}

module.exports.debugLog = function () {
  if (!config.server.debug) return

  console.log.apply(null, arguments)
}

module.exports.errorLog = function (errObj, specify) {
  if (specify) console.error(specify)
  console.error(errObj)

  if (errObj.stack) {
    console.error(errObj.stack)
  }
}

module.exports.printRuntimeStats = function (runtimeStats, connections) {
  if (!config.server.printStats) return

  let date = new Date()
  let users = 0
  let servers = 0
  let serversPerUser = 0

  for (let uid in connections) {
    let ca = connections[uid]
    users += 1
    for (let snam in ca) {
      if (!snam) continue
      if (snam === 'host') continue
      servers += 1
    }
  }

  if (users !== 0) { // Don't divide by zero lmao
    serversPerUser = servers / users
  }

  console.log(date,
    util.format('- Currently connected users: %d. ' +
    'IRC server connections: %d. ' +
    'Average servers per user: %f. ' +
    'Total connections made: %d. ' +
    'Uptime: %d s;', users, servers, serversPerUser, runtimeStats.connectionsMade, process.uptime())
  )
}
